/* Iterating through multibyte strings: macros for multi-byte encodings.
   Copyright (C) 2001, 2005, 2007 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/* Written by Bruno Haible <bruno@clisp.org>.  */

/* The macros in this file implement forward iteration through a
   multi-byte string.  They are to be used when MB_CUR_MAX > 1.  The same
   macros are defined in mbiter_8bit.h for iteration through a string
   in an 8-bit encoding, for use when MB_CUR_MAX == 1.

   With these macros, an iteration loop that looks like

      char *iter;
      for (iter = buf; iter < buf + buflen; iter++)
        {
          do_something (*iter);
        }

   becomes

      MBI_ITERATOR iter;
      for (MBI_INIT (iter, buf, buflen); MBI_AVAIL (iter); MBI_ADVANCE (iter))
        {
          do_something (MBI_CUR_PTR (iter), MB_LEN (MBI_CUR (iter)));
        }

   The benefits of these macros over plain use of mbrtowc are:
   - Generation of a fully efficient 8-bit version from the same code,
     without code duplication.
   - Handling of invalid multibyte sequences is possible without
     making the code more complicated, while still preserving the
     invalid multibyte sequences.

   MBI_ITERATOR
     expands to a type usable for variable declarations.

   MBI_INIT (iter, startptr, length)
     initializes the iterator, starting at startptr and crossing length bytes.

   MBI_AVAIL (iter)
     returns true if there are more multibyte chracters available before
     the end of string is reached. In this case, MBI_CUR (iter) is
     initialized to the next multibyte chracter.

   MBI_ADVANCE (iter)
     advances the iterator by one multibyte character.

   MBI_CUR (iter)
     returns the current multibyte character, of type MB_CHAR.  All the macros
     defined in mbchar_multi.h can be used on it.

   MBI_CUR_PTR (iter)
     return a pointer to the beginning of the current multibyte character.

   MBI_RELOC (iter, ptrdiff)
     relocates iterator when the string is moved by ptrdiff bytes.

   MBI_COPY (destiter, srciter)
     copies srciter to destiter.

   Here are the function prototypes of the macros.

   extern void		MBI_INIT (MBI_ITERATOR iter,
				  const char *startptr, size_t length);
   extern bool		MBI_AVAIL (MBI_ITERATOR iter);
   extern void		MBI_ADVANCE (MBI_ITERATOR iter);
   extern MB_CHAR	MBI_CUR (MBI_ITERATOR iter);
   extern const char *	MBI_CUR_PTR (MBI_ITERATOR iter);
   extern void		MBI_RELOC (MBI_ITERATOR iter, ptrdiff_t ptrdiff);
   extern void		MBI_COPY (MBI_ITERATOR new, const MBI_ITERATOR old);
 */

#ifndef _MBITER_MULTI_H
#define _MBITER_MULTI_H 1

#include "mbiter.h"

#endif /* _MBITER_MULTI_H */
/* Now the macros that will be undefined in mbiter_undef.h.  */

#include "mbchar_multi.h"

/* Iteration macros.  */
#define MBI_ITERATOR mbi_iterator_t
#define MBI_INIT(iter, startptr, length) mbi_init (iter, startptr, length)
#define MBI_AVAIL(iter) mbi_avail (iter)
#define MBI_ADVANCE(iter) mbi_advance (iter)

/* Access to the current character.  */
#define MBI_CUR(iter) mbi_cur (iter)
#define MBI_CUR_PTR(iter) mbi_cur_ptr (iter)

/* Relocation.  */
#define MBI_RELOC(iter, ptrdiff) mbi_reloc (iter, ptrdiff)

/* Copy.  */
#define MBI_COPY(new, old) mbi_copy (&(new), &(old))
