/* Iterating through multibyte strings: macros for multi-byte encodings.
   Copyright (C) 2001, 2005, 2007 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/* Written by Bruno Haible <bruno@clisp.org>.  */

/* The macros in this file implement forward iteration through a
   multi-byte string.  They are to be used when MB_CUR_MAX > 1.  The same
   macros are defined in mbuiter_8bit.h for iteration through a string
   in an 8-bit encoding, for use when MB_CUR_MAX == 1.

   With these macros, an iteration loop that looks like

      char *iter;
      for (iter = buf; iter < buf + buflen; iter++)
        {
          do_something (*iter);
        }

   becomes

      MBUI_ITERATOR iter;
      for (MBUI_INIT (iter, buf, buflen); MBUI_AVAIL (iter); MBUI_ADVANCE (iter))
        {
          do_something (MBUI_CUR_PTR (iter), MB_LEN (MBUI_CUR (iter)));
        }

   The benefits of these macros over plain use of mbrtowc are:
   - Generation of a fully efficient 8-bit version from the same code,
     without code duplication.
   - Handling of invalid multibyte sequences is possible without
     making the code more complicated, while still preserving the
     invalid multibyte sequences.

   Compared to mbiter_multi.h, the macros here don't need to know the string's
   length a-priori.  The downside is that at each step, the look-ahead
   that guards against overrunning the terminating '\0' is more expensive.
   The MBUI_* macros are therefore suitable when there is a high probability
   that only the first few multibyte characters need to be inspected.
   Whereas the MBI_* macros are better if usually the iteration runs
   through the entire string.

   MBUI_ITERATOR
     expands to a type usable for variable declarations.

   MBUI_INIT (iter, startptr)
     initializes the iterator, starting at startptr.

   MBUI_AVAIL (iter)
     returns true if there are more multibyte chracters available before
     the end of string is reached. In this case, MBUI_CUR (iter) is
     initialized to the next multibyte chracter.

   MBUI_ADVANCE (iter)
     advances the iterator by one multibyte character.

   MBUI_CUR (iter)
     returns the current multibyte character, of type MB_CHAR.  All the macros
     defined in mbchar_multi.h can be used on it.

   MBUI_CUR_PTR (iter)
     return a pointer to the beginning of the current multibyte character.

   MBUI_RELOC (iter, ptrdiff)
     relocates iterator when the string is moved by ptrdiff bytes.

   MBUI_COPY (destiter, srciter)
     copies srciter to destiter.

   Here are the function prototypes of the macros.

   extern void		MBUI_INIT (MBUI_ITERATOR iter, const char *startptr);
   extern bool		MBUI_AVAIL (MBUI_ITERATOR iter);
   extern void		MBUI_ADVANCE (MBUI_ITERATOR iter);
   extern MB_CHAR	MBUI_CUR (MBUI_ITERATOR iter);
   extern const char *	MBUI_CUR_PTR (MBUI_ITERATOR iter);
   extern void		MBUI_RELOC (MBUI_ITERATOR iter, ptrdiff_t ptrdiff);
   extern void		MBUI_COPY (MBUI_ITERATOR new, const MBUI_ITERATOR old);
 */

#ifndef _MBUITER_MULTI_H
#define _MBUITER_MULTI_H 1

#include "mbuiter.h"

#endif /* _MBUITER_MULTI_H */
/* Now the macros that will be undefined in mbuiter_undef.h.  */

#include "mbchar_multi.h"

/* Iteration macros.  */
#define MBUI_ITERATOR mbui_iterator_t
#define MBUI_INIT(iter, startptr) mbui_init (iter, startptr)
#define MBUI_AVAIL(iter) mbui_avail (iter)
#define MBUI_ADVANCE(iter) mbui_advance (iter)

/* Access to the current character.  */
#define MBUI_CUR(iter) mbui_cur (iter)
#define MBUI_CUR_PTR(iter) mbui_cur_ptr (iter)

/* Relocation.  */
#define MBUI_RELOC(iter, ptrdiff) mbui_reloc (iter, ptrdiff)

/* Copy.  */
#define MBUI_COPY(new, old) mbui_copy (&(new), &(old))
